package org.ow2.bonita.example.aw;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.Map;

import javax.security.auth.login.LoginContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;

@SuppressWarnings("serial")
public class AWAction extends HttpServlet {

  static final String PROCESS_ID = "ApprovalWorkflow";

  /**
   * Constructor of the object.
   */
  public AWAction() {
    super();
  }

  public void destroy() {
    super.destroy();
  }

  public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

    final String userName = request.getParameter("username");
    final String password = request.getParameter("password");
    final String action = request.getParameter("Action");
    
    response.setContentType("text/html");
    PrintWriter out = response.getWriter();
    AWCommon.writeStart(out);
    
    try {
      final LoginContext loginContext = new LoginContext("Bonita", new SimpleCallbackHandler(userName, password));
      loginContext.login();
      
      final RuntimeAPI runtimeAPI = AccessorUtil.getRuntimeAPI();
      final QueryRuntimeAPI queryRuntimeAPI = AccessorUtil.getQueryRuntimeAPI();
      final ManagementAPI managementAPI = AccessorUtil.getManagementAPI();
      final QueryDefinitionAPI queryDefinitionAPI = AccessorUtil.getQueryDefinitionAPI();
      
      if (action.equals("Deploy")) {
        
        final File file = new File(request.getParameter("BarFile"));
        if (!file.exists()) {
          throw new Exception("The given file : " + file 
              + " does not exist. Are you sure you have generated the bar file ?");
        }
        byte[] bar = Misc.getAllContentFrom(file);
        Map<String, org.ow2.bonita.facade.def.majorElement.ProcessDefinition> map = managementAPI.deployBar(bar);
        ProcessDefinition process = map.get(PROCESS_ID);
        out.println("<br>Informations about deployment :");
        out.println("<br>Process Name : " + process.getName());
        out.println("<br>Process ID : " + process.getProcessId());
        out.println("<br>Process UUID : " + process.getUUID());

      } else if (action.equals("Start")) {
        org.ow2.bonita.facade.def.majorElement.ProcessDefinition processDef = queryDefinitionAPI.getLastProcess(PROCESS_ID);
        
        final ProcessInstanceUUID instanceUUID = runtimeAPI.instantiateProcess(processDef.getUUID());
        final Collection<ActivityInstance<TaskInstance>> activities = queryRuntimeAPI.getTaskList(instanceUUID, TaskState.READY);
        if (activities.isEmpty()) {
          throw new Exception("No task found? Bad User?");
        }
        for (ActivityInstance<TaskInstance> activity : activities) {
          TaskUUID taskUUID = activity.getBody().getUUID();
          runtimeAPI.startTask(taskUUID, true);
          out.println("<br><br>Starting task: " + taskUUID);
          runtimeAPI.finishTask(taskUUID, true);
          out.println("<br><br>Task " + taskUUID + " finished.");
        }

      } else if (action.equals("Undeploy")) {
        
        final ProcessDefinition processDef = queryDefinitionAPI.getLastProcess(PROCESS_ID);
        PackageDefinitionUUID packageDefinitionUUID = processDef.getPackageDefinitionUUID();
        managementAPI.undeploy(packageDefinitionUUID);
        out.println("<br>Package with UUID " + packageDefinitionUUID + "  was succesully undeployed");
        
      }

      loginContext.logout();
    } catch (Exception e1) {
      StackTraceElement[] stackElements = e1.getStackTrace();
      out.println("<div align='left'>");
      out.println(e1.getMessage() + "<br>");
      for (StackTraceElement stackElement : stackElements) {
        out.println("  at " + stackElement + "<br>");
      }
      out.println("</div>");
      e1.printStackTrace();
    }
    out.println("<br><br><br>");
    out.println("<form  name='form' method='get' action='" + AWCommon.ACTIONS_URL + "'>");
    out.println("<input type='submit' value='BACK TO ACTIONS' name='back'>");
    out.println("<input type='hidden' name='username' value='" + userName + "'>");
    out.println("<input type='hidden' name='password' value='" + password + "'>");
    out.println("</form>");
    
    AWCommon.writeEnd(out);
    out.flush();
    out.close();
  }

  public void init() throws ServletException {

  }

}

